"""
Module containing functionallity for resolving predicate names from
third party sources.
"""
import logging
import requests
import rdflib
import StringIO

LABEL_CANDIDATES = ['http://www.w3.org/2000/01/rdf-schema#label']

# All providers don't use the correct way of supplying the rdf
# so we use this to map the inconsistencies.
URL_REMAP = {
    'http://schema.theodi.org/odrs': 'http://schema.theodi.org/odrs/index.ttl',
}

# All providers don't use xml format so here we map other formats
FORMATS = {
    'http://schema.theodi.org/odrs': 'n3',
}


class PredicateResolver(object):
    """
    Class for resolving a human readable version of an RDF predicate
    """

    def __init__(self):
        self._resolved = {}
        self._parsed = []

    def _get_from_cache(self, url, language):
        """
        Get the resolved name from the cache.
        :param url: url of the predicate
        :param language: language to use
        :return:
        """
        if url in self._resolved:
            if language in self._resolved[url]:
                # Prefered language found
                return self._resolved[url][language]
            elif 'en' in self._resolved[url]:
                # Fallback to english
                return self._resolved[url]['en']

    def resolve(self, url, language):
        """
        Try to get a human readable form of an RDF predicate.
        :param url: url of the predicate
        :param language: language to use
        :return:
        """
        # Do we have it cached?
        cached = self._get_from_cache(url, language)
        if cached:
            return cached

        # Fetch it from the remote specification
        self.get_rdf(url)

        # It should now be present in the cache
        return self._get_from_cache(url, language)

    def get_rdf(self, url):
        """
        :param url:
        :return:
        """
        # Most providers use # or / for delimiting the identifier from the
        # actual download url
        if '#' in url:
            url = url.rsplit('#', 1)[0]
        elif '/' in url:
            url = url.rsplit('/', 1)[0]

        # Have we already downloaded and this url?
        if url in self._parsed:
            return

        headers = {'Accept': 'application/rdf+xml'}

        # Some URLs we've encountered do not provide xml versions of the rdf
        if url in FORMATS:
            rdf_format = FORMATS[url]
        else:
            rdf_format = 'xml'

        # Some URLs also do not follow the standard way of requesting a
        # the resource on a different format. They do however provide
        # the resource but at a differnt URL.
        if url in URL_REMAP:
            url = URL_REMAP[url]

        logging.info('Downloading %s', url)
        try:
            resp = requests.get(url, headers=headers)
        except Exception as err:  # pylint: disable=W0703
            # We want to catch all exceptions here
            logging.warning('Unable to download %s. %s', url, err.message)
            return

        file_obj = StringIO.StringIO()
        file_obj.write(resp.text.encode('utf-8'))
        file_obj.seek(0)

        graph = rdflib.Graph()
        try:
            graph.load(file_obj, format=rdf_format)
        except Exception as err:  # pylint: disable=W0703
            # We want to catch all exceptions here
            logging.warning('Unable to parse file: %s. %s', url, err.message)
        for subj, pred, obj in graph:

            if isinstance(obj, rdflib.Literal) and unicode(pred) in LABEL_CANDIDATES:
                subj = unicode(subj)
                if subj not in self._resolved:
                    self._resolved[subj] = {}
                if obj.language:
                    language = obj.language
                else:
                    language = 'en'
                self._resolved[subj][language] = unicode(obj.value).title()

        self._parsed.append(url)



"""
Module containing code for uploading a human readable version of
RDF data to a CKAN instance.
"""

import argparse
import os
import tempfile
import requests

import ckanapi
from rdfconv.converter import RDFtoHTMLConverter


class RDFLoader(object):
    """
    Class able to convert a single nodes in an RDF file to
    a simpler representation.
     Args:
            filename: filename of the RDF file
    """
    def __init__(self, rdf_path):
        self.conv = RDFtoHTMLConverter()
        self.conv.skip_links = True
        self.conv.load_file(rdf_path)
        self.nodes = self.conv.get_nodes('en')

    def convert_node(self, rdf_about):
        """
        Convert a single node from the RDF file
        Args:
            rdf_about: unique identifier for the node

        Returns:
            An intermediate format, essentially a nested dictionary,
            of the node.
        """
        # Find the node we want
        node = None
        for cur_node in self.nodes:
            if cur_node['rdf_about'] == rdf_about:
                node = cur_node
                break

        if not node:
            # Skip the node if it's not found
            return

        return node


class CKANUploader(object):
    """
    Class for updating datasets in CKAN that have a connection to a RDF
    dataset.
    """

    def __init__(self, ckan_url, api_key):
        """
        Create the object
        Args:
            ckan_url: URL to the CKAN instance you want to update
            api_key: API key to a user that has write access to the datasets
                     you want to update

        """
        if not ckan_url.startswith(('http://', 'https://')):
            ckan_url = 'http://' + ckan_url
        self.api = ckanapi.RemoteCKAN(ckan_url, apikey=api_key)
        self._fetch_datasets()

    def _fetch_datasets(self):
        """
        Fetch all datasets that have a connection to an RDF dataset.

        """
        datasets = self.api.action.package_list()
        self.mapping = {}
        for name in datasets:
            dataset = self.api.action.package_show(id=name)
            if 'dcat_about' in dataset:
                self.mapping[dataset['dcat_about']] = name

    def update_datasets(self, rdf_path):
        """
        Update all datasets found when creating the object
        with data from given file
        Args:
            rdf_path: path to an RDF file
        """
        loader = RDFLoader(rdf_path)
        for rdf_about, ckan_name in self.mapping.iteritems():
            node = loader.convert_node(rdf_about)
            # Node wasn't found in the RDF file so we skip it
            if not node:
                continue
            extras = self._convert_to_extras(node['attributes'])
            self._update_dataset(ckan_name, rdf_about, extras)

    def _update_dataset(self, name, rdf_about, extras):
        """
        Update a single dataset
        Args:
            name: name or id of the dataset
            rdf_about: unique identifier for the RDF node
            extras: dcat data
        """
        current_info = self.api.action.package_show(id=name)

        # We need to send the dcat id as an extra
        extras.append({
            'key': '_dcat_about_',
            'value': rdf_about
        })
        current_info['extras'] += extras

        # We also need to remove the old dcat stuff
        del current_info['dcat_fields']
        del current_info['dcat_about']

        self.api.action.package_update(**current_info)

    @staticmethod
    def _convert_to_extras(attributes):
        """
        Args:
            attributes: RDF attributes

        Returns:
            A list of extras formatted for insertion into CKAN.
        """
        extras = []
        for attribute in attributes:
            objs = []
            for obj in attribute['objs']:
                if 'link' in obj:
                    objs.append(unicode(obj['link']) + ';' + obj['title'])
                else:
                    objs.append(';' + obj['title'])

            extras.append({
                'key': '_dcat_field_' + unicode(attribute['pred_link']) +
                       ';' + attribute['pred_title'],
                'value': ';'.join(objs)
            })

        return extras


def main():
    """
    Run the uploader
    """
    # Handle arguments
    parser = argparse.ArgumentParser(
        description='CKAN RDF uploader. Converts data from a RDF file '
                    'and load it into CKAN for use with the rdf-to-html '
                    'CKAN extension',)
    parser.add_argument('ckan_url', metavar='CKAN_URL', type=str,
                        help='URL to ckan instance')
    parser.add_argument('api_key', metavar='API_KEY', type=str,
                        help='API key for a user with write access to the '
                             'datasets you want to update')
    parser.add_argument('rdf_file', metavar='RDF_FILE', type=str,
                        help='Path to the RDF file')

    args = parser.parse_args()

    temp_file = False
    if args.rdf_file.startswith(('http://', 'https://')):
        rdf_path = download_file(args.rdf_file)
        temp_file = True
    else:
        rdf_path = args.rdf_file

    uploader = CKANUploader(args.ckan_url, args.api_key)
    uploader.update_datasets(rdf_path)

    if temp_file:
        os.remove(rdf_path)


def download_file(url):
    """
    Download an RDF file from an URL
    Args:
        url: URL to the file

    Returns: filename of the downloaded file
    """
    dcat = requests.get(url)

    _, filename = tempfile.mkstemp(suffix='.rdf')
    with open(filename, 'w') as file_obj:
        file_obj.write(dcat.content)
    return filename

if __name__ == '__main__':
    main()



"""
Contains code related to outputing HTML
"""
import os
import codecs
import urllib2
import re
from datetime import datetime

from rdflib.term import URIRef, BNode, Literal
from django.template import Context
from django.template.loader import get_template
from django.conf import settings

from rdfconv.predicate import PredicateResolver


RDF_ABOUT = URIRef(u'http://www.w3.org/1999/02/22-rdf-syntax-ns#about')

CATALOG = URIRef(u'http://www.w3.org/ns/dcat#Catalog')
DATASET = URIRef(u'http://www.w3.org/ns/dcat#Dataset')
DISTRIBUTION = URIRef(u'http://www.w3.org/ns/dcat#Distribution')

OBJ_ORDER = [CATALOG, DATASET, DISTRIBUTION]


class HtmlConverter(object):
    """
    Class that converts a dictionary of RdfObjects into HTML
    """

    def __init__(self, rdf_objects, ns_mgr):
        self.objects = rdf_objects
        self._ns_mgr = ns_mgr

        # Predicate resolver
        self._pred_res = PredicateResolver()

        # Init templates

        if not settings.configured:
            base_dir = os.path.dirname(os.path.abspath(__file__))
            settings.configure(
                TEMPLATE_DIRS=(os.path.join(base_dir, 'templates'),),
                TEMPLATE_LOADERS=("django.template.loaders.filesystem.Loader",),
                TEMPLATE_DEBUG=True)

        self.skip_internal_links = False
        self.skip_literal_links = False

    def build_node_dict(self, language):
        """
        Build list of nested dictionaries to use as an intermediate
        format before rendering the HTML.

        :param language: language to convert to
        :returns: dictionary with nodes
        """
        objects = []
        for rdf_type in OBJ_ORDER:
            for obj in self.objects.values():
                if obj.type == rdf_type:
                    objects.append(obj)

        for obj in self.objects.values():
            if obj not in objects:
                objects.append(obj)

        nodes = []
        for obj in objects:
            node_dict = {'node_id': obj.fragment,
                         'rdf_about': obj.id}
            summary = self._format_summary(obj, language)
            attributes = self._format_node(obj, language)

            node_dict.update(summary)
            node_dict.update({'attributes': attributes})

            nodes.append(node_dict)
        return nodes

    def output_html(self, path, language):
        """
        Output each node to a separate file per language
        :param path:
        :param language:
        :return:
        """
        nodes = self.build_node_dict(language)

        # TODO: We might want to add the timezone here
        date = datetime.now().strftime('%Y-%m-%d %H:%M')

        context = Context({'nodes': nodes,
                           'date': date})

        main_template = get_template('main.html')
        out = main_template.render(context)

        with codecs.open(path, 'w', 'utf-8') as output_file:
            output_file.write(out)

    def _format_summary(self, rdf_obj, language):
        """
        Generate a summary for an RDF node
        """
        # Try to find something to use as a title and a description
        title = rdf_obj.get_title(language)
        desc = rdf_obj.get_description(language)
        rdf_type = rdf_obj.type

        out = {}
        if title:
            out['title'] = title

        if rdf_type:
            # Try to resolve the type
            label = self._pred_res.resolve(rdf_type.toPython(), language)
            if label:
                out['rdf_type'] = label
            else:
                out['rdf_type'] = rdf_type

        if desc:
            out['desc'] = desc

        return out

    def _format_node(self, rdf_obj, language, include=None):
        """
        Returns a single node as a table with all:
        * Literals - formatted with the desired language
        * BNodes   - formatted as links
        * URIRefs  - formatted as links
        """
        if not include:
            include = sorted(rdf_obj.attributes.keys())

        attributes = []
        # Add the RDF id att the top
        pred_link, pred_title = self._format_uriref(RDF_ABOUT, language, skip_local=self.skip_internal_links)
        obj_link, obj_title = self._format_uriref(rdf_obj.id, language, skip_local=True)
        attributes.append({
            'pred_link': pred_link,
            'pred_title': 'About',
            'objs': [{'title': obj_title,
                      'link': obj_link}],
        })

        for pred in sorted(include):
            try:
                obj_list = rdf_obj.attributes[pred]
            except KeyError:
                # Skip the desired attributes if they are not present
                continue

            pred_link, pred_title = self._format_uriref(pred, language, skip_local=self.skip_literal_links)
            # Try to resolve the predicate to a more human readable format
            label = self._pred_res.resolve(pred_link, language)
            if label:
                pred_title = label

            objs = []
            if obj_list and isinstance(obj_list[0], Literal):
                literals = format_literal(obj_list, language, self.skip_literal_links)
                objs.append({'title': u' '.join(literals)})
            else:
                # Get the other objects and sort them based on their title
                new_list = []
                for obj in obj_list:
                    if isinstance(obj, URIRef):
                        new_list.append(self._format_uriref(obj, language, self.skip_internal_links))
                    elif isinstance(obj, BNode):
                        new_list.append(self._format_bnode(obj, language, self.skip_internal_links))

                new_list = sorted(new_list, key=lambda t: t[1])

                for link, title in new_list:
                    objs.append({'link': link,
                                 'title': title})

            attributes.append({
                'pred_link': pred_link,
                'pred_title': pred_title,
                'objs': objs,

            })

        # Add show more button
        return attributes

    def _format_uriref(self, uri_ref, language, skip_local=False):
        """
        Return the HTML representation of a URIRef
        """
        # Does it point to a local asset?
        local_ref = unicode(uri_ref)
        if local_ref in self.objects:
            return self._format_bnode(uri_ref, language, skip_local)

        # It seems that some URIRefs get normalized with a '<' and a '>'
        # at the start/end of the string.
        # We need to remove it
        norm = self._ns_mgr.normalizeUri(uri_ref).strip()

        if norm[0] == '<':
            norm = norm[1:]
        if norm[-1] == '>':
            norm = norm[:len(norm)-1]
        if norm[-1] == '/':
            norm = norm[:len(norm)-1]

        return uri_ref, norm

    def _format_bnode(self, bnode, language, skip_local=False):
        """
        Return the HTML representation of a BNonde
        """
        # Check if we can get a title for the bnode
        rdf_id = unicode(bnode)

        link = self._get_fragment_link(rdf_id)
        if link:
            title = self.objects[rdf_id].get_title(language)
            if not skip_local:
                return link, title
            else:
                return rdf_id, title

        return None, None

    def _get_fragment_link(self, rdf_id):
        """
        Get a link to a file containing an RDF node. Returns none if the
        node could not be found in the current context
        """
        try:
            return '#' + self.objects[unicode(rdf_id)].fragment

        except KeyError:
            return None


# region Literal formatting

# Characters allowed in an URL according to RDF 3986
LINK_REGEX = re.compile(r'(http://[ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789\-._~:/?#\[\]@!$&\'()*+,;=%%]+)')
TO_WHITESPACE = re.compile(r'[_-]')


def format_literal(literals, language, skip_link=False):
    """
    Return the HTML representation of one or more Literals.
    First try to get the specified language and if it does not exist, get
    the literals without a language tag. Finally, return literals of
    another language if there are not better match
    """
    same_lang = []
    no_lang = []
    other_lang = []
    if not literals:
        return ''

    for literal in literals:
        if not skip_link:
            value = _add_html_links(literal.value)
        else:
            value = literal.value
        if literal.language == language:
            same_lang.append(value)
        elif not literal.language:
            no_lang.append(value)
        else:
            other_lang.append(value)

    if same_lang:
        return sorted(same_lang)
    if no_lang:
        return sorted(no_lang)
    if other_lang:
        return sorted(other_lang)


def _add_html_links(string):
    """
    Find anything that looks like a hyperlink and convert it to an actual
    HTML link.
    """
    matches = LINK_REGEX.findall(string)
    for url in matches:
        # There is a special case when the URL is entered between parenthesis.
        # We want to remove the last character if it's a closing parenthesis
        # and if the url is preceeded by the matching opening parenthesis.
        if url[-1] == ')':
            char = _get_preceding_character(string, url)
            if char and char == '(':
                url = url[:-1]

        # If the last character is a ".", it's most likely used to end a
        # sentence rather than as part of the url. urls ending with "." are
        # not valid anyhow.
        if url[-1] == '.':
            url = url[:-1]

        display_name = url
        splits = display_name.rsplit('/', 1)
        if display_name.count('/') > 2 and len(splits) >= 2:
            display_name = splits[1]

        # Remove the URL encoding
        display_name = urllib2.unquote(display_name)
        # We want the string in Unicode, not UTF-8, beacuse django seems to
        # like it this way. Encoding the string as latin-1 and decoding it
        # again seems to produce a pure unicode string.
        display_name = display_name.encode('latin-1').decode('utf-8')

        # We also want to remove underscores and such
        display_name = TO_WHITESPACE.sub(' ', display_name)

        html_link = _make_link(url, display_name)
        string = string.replace(url, html_link)

    return string


def _make_link(url, display_name):
    """
    Make a HTML link from an url and a display name
    """
    return u'<a href=%s target="_blank">%s</a>' % (url, display_name)


def _get_preceding_character(string, sub_string):
    """
    Search a string for a substring and return the single character before the
    sub string
    """
    pos = string.find(sub_string) - 1
    if pos != -1:
        return string[pos]

# endregion



"""
Class containing code related to RDF objects
"""

import hashlib
from rdfconv.utils import get_attribute
from rdfconv.html import format_literal

# Namespaces
TYPE = u'http://www.w3.org/1999/02/22-rdf-syntax-ns#type'
TITLE = u'http://purl.org/dc/terms/title'
FOAF_NAME = u'http://xmlns.com/foaf/0.1/name'
LABEL = u'http://www.w3.org/2000/01/rdf-schema#label'
VCARD_NAME = u'http://www.w3.org/2006/vcard/ns#fn'
DESC = u'http://purl.org/dc/terms/description'

# Attributes with these namespaces are candidates for the summary
# title/description
TITLE_CANDIDATES = [TITLE, FOAF_NAME, LABEL, VCARD_NAME]
DESC_CANDIDATES = [DESC]


class RdfObject(object):
    """
    Class representing an rdf object.
    Contains methods for easily accessing common attributes
    """

    def __init__(self, id, attributes, ns_mgr=None):

        # Rdf type
        self.type = None

        # Rdf id
        self.id = id

        # Fragment id
        self.fragment = hashlib.md5(id).hexdigest()

        # List of potential titles and descriptions
        self.title = None
        self.description = None

        # Other attributes
        self.attributes = attributes

        # Try to find a type
        types = get_attribute(attributes, [TYPE])
        if types:
            self.type = types[0]

        # Get a list of potential titles and descriptions
        self.title = get_attribute(attributes, TITLE_CANDIDATES)
        self.description = get_attribute(attributes, DESC_CANDIDATES)

        # Reference to a ns manager
        self._ns_mgr = ns_mgr

    def __repr__(self):
        return self.id

    def __eq__(self, other):
        return self.id == other.id

    def get_title(self, language):
        """
        Gets the title of the RDF object
        """
        candidates = format_literal(self.title, language)
        if candidates:
            return candidates[0]
        else:
            return self.id

    def get_description(self, language):
        """
        Gets the description of the RDF object
        """
        candidates = format_literal(self.description, language)
        if candidates:
            return candidates[0]
        return ''

    def get_canoical_type(self):
        """
        Gets the shortened type of the RDF object
        """
        if not self.type:
            return ''
        norm = self._ns_mgr.normalizeUri(self.type).strip()

        if norm[0] == '<':
            norm = norm[1:]
        if norm[-1] == '>':
            norm = norm[:len(norm)-1]
        if norm[-1] == '/':
            norm = norm[:len(norm)-1]
        return norm

    def get_sort_tuple(self, language):
        """
        Get the sort order of the object
        """
        return (self.get_canoical_type(), self.get_title(language))



"""
Contains various utils used by the converter
"""


def get_attribute(node, candidates):
    """
    Gets an attribute from a list of candidates
    """
    if not isinstance(candidates, list):
        candidates = [candidates]
    for candidate in candidates:
        if candidate in node:
            return node[candidate]
    return None


def get_file(name, language):
    """
    Format a filename based on a name and a language
    """
    return '%s.html.%s' % (name, language)



from setuptools import setup, find_packages

try:
    import pypandoc
    long_desc = pypandoc.convert('README.md', 'rst')
except ImportError:
    with open('README.md') as readme:
        long_desc = readme.read()


setup(
    name='rdf-to-html',
    version='1.1.0',
    description='RDF to HTML converter',
    license='EUPL v1.1',
    long_description=long_desc,

    # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 2.7",
    ],
    keywords=['Open Data', 'Linked Open Data', 'RDF', 'DCAT'],
    author='Kim Nilsson',
    author_email='kim.nilsson@dohi.se',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'rdflib>=4.2.1',
        'django>=1.8.6',
        'requests>=2.8.1',
        'pyinotify>=0.9.6',
        'ckanapi',
    ],
    entry_points={
        'console_scripts': [
            'rdf-to-html=rdfconv.main:main',
            'ckan-uploader=ckan_uploader.uploader:main'
        ],
    },
)



"""
This module contains code for converting RDF-files into HTML
"""
import os
import logging
import shutil
from collections import OrderedDict

import rdflib
from rdflib.term import Literal

from rdfconv.utils import get_file
from rdfconv.html import HtmlConverter
from rdfconv.objects import RdfObject

if not logging:
    # rdflib requires a logger to be setup
    logging.basicConfig()


class Error(Exception):
    """
    Base class for exceptions in this module
    """
    pass


class LanguageError(Error):
    """
    Raised when the specified languages differ from the actual languages
    encountered in the RDF file.
    """
    def __init__(self, filename, specified, actual):
        super(LanguageError, self).__init__()
        error = 'Languages encountered in RDF file differ from specified ' \
                'languages %s.\n\t' +\
                'Specified: %s\n\t' +\
                'Actual:    %s'

        specified = ','.join(sorted(specified))
        actual = ','.join(sorted(actual))

        self.msg = error % (filename, specified, actual)

    def __str__(self):
        return self.msg


class RDFtoHTMLConverter(object):
    """
    Class representing a RDF to HTML converter
    """

    def __init__(self, languages=None):
        if not languages:
            languages = ['all']

        # References to the underlying graph
        self._graph = None
        self._ns_mgr = None

        # Dictionary representation of the RDF file
        self.objects = OrderedDict()

        # Keep track of all languages seen in the RDF
        self.languages = set()

        # Keep track of desired languages
        self.specified_languages = set(languages)

        # The currently loaded file
        self.input_file = None

        self._skip_links = False

    @property
    def skip_links(self):
        """
        Should the converter skip converting stuff that looks like links
        to actual HTML links?
        :return:
        """
        return self._skip_links

    @skip_links.setter
    def skip_links(self, value):
        """
        Tells the converter where to skip converting what looks like
        links to actual HTML links
        :param value:
        :return:
        """
        self._skip_links = value

    def load_file(self, filename):
        """
        Read RDF data from file
        """
        self.input_file = os.path.basename(filename)

        # Load graph from file
        self._graph = rdflib.Graph()
        self._graph.load(filename, format='application/rdf+xml')

        # Easy access to namespace manager
        self._ns_mgr = self._graph.namespace_manager

        # Build dictionary
        rdf_dict = {}
        for subj, pred, obj in self._graph:

            if subj.toPython() not in rdf_dict:
                rdf_dict[subj.toPython()] = {}
            if pred.toPython() not in rdf_dict[subj.toPython()]:
                rdf_dict[subj.toPython()][pred.toPython()] = []

            rdf_dict[subj.toPython()][pred.toPython()].append(obj)
            if isinstance(obj, Literal):
                if obj.language:
                    # Literals can have a language tag,
                    # Keep track of all languages encountered
                    self.languages.add(obj.language)

        # Generate objects
        objects = []
        for key, value in rdf_dict.iteritems():
            obj = RdfObject(key, value, self._ns_mgr)
            objects.append(obj)

        # Sort them by type -> title
        self.objects = OrderedDict()
        for obj in sorted(objects, key=lambda x: x.get_sort_tuple('en')):
            self.objects[obj.id] = obj

        self._validate_languages()

    def output_html(self, folder):
        """
        Output one file per language encountered in the rdf file
        """
        if not os.path.exists(folder):
            os.mkdir(folder)
        elif not os.path.isdir(folder):
            logging.error('Could not write output. %s is not a directory.',
                          folder)
            exit(1)

        # Move script and style files
        base_dir = os.path.dirname(os.path.abspath(__file__))
        shutil.copy(os.path.join(base_dir, 'includes/style.css'), folder)
        shutil.copy(os.path.join(base_dir, 'includes/rdfconv.js'), folder)

        html_conv = HtmlConverter(self.objects, self._ns_mgr)
        if self.skip_links:
            html_conv.skip_literal_links = True
            html_conv.skip_internal_links = True

        # Assume english if no language was encountered
        if not self.languages:
            self.languages.add('en')
        for language in self.languages:
            filename = os.path.splitext(self.input_file)[0]
            filename = get_file(filename, language)
            path = os.path.join(folder, filename)
            html_conv.output_html(path, language)

    def get_nodes(self, language):
        """
        Get the nodes parsed from the RDF file

        This is used before running the actual HTML conversion
        and can be used by other software that wants to render
        their own version of the data.
        :param language:
        :return: a list of nested dictionaries with information
                 about each node in the RDF file
        """
        html_conv = HtmlConverter(self.objects, self._ns_mgr)
        if self.skip_links:
            html_conv.skip_literal_links = True
            html_conv.skip_internal_links = True
        return html_conv.build_node_dict(language)

    def _validate_languages(self):
        """
        Make sure the languages specified by the user are the same as those
        encountered in the RDf file.
        """
        if 'all' in self.specified_languages:
            return

        if self.languages != self.specified_languages:
            raise LanguageError(self.input_file,
                                self.specified_languages,
                                self.languages)



"""
Main entry point
"""

import argparse
import logging
import sys
from rdfconv.converter import RDFtoHTMLConverter, LanguageError
import pyinotify


class EventHandler(pyinotify.ProcessEvent):
    """
    Class handling notifications when a watched file is changed
    """
    def __init__(self, output_folder, languages):
        super(EventHandler, self).__init__()
        self.output_folder = output_folder
        self.languages = languages

    def process_default(self, event):
        logging.info('%s changed', event.path)
        run(event.path, self.output_folder, self.languages)


def run(input_file, output_folder, languages='all'):
    """
    Run the RDF converter
    """
    try:
        logging.info('Converting %s', input_file)
        rdf_conv = RDFtoHTMLConverter(languages)
        rdf_conv.load_file(input_file)
        rdf_conv.output_html(output_folder)
        logging.info('Finished converting %s', input_file)
    except LanguageError as err:
        logging.error('Skipped file %s: %s', input_file, err)


def watch(input_files, output_folder, languages='all'):
    """
    Setup watching of given files
    """
    handler = EventHandler(output_folder, languages)
    watch_manager = pyinotify.WatchManager()
    notifier = pyinotify.Notifier(watch_manager, handler)
    for input_file in input_files:
        watch_manager.add_watch(input_file, pyinotify.IN_MODIFY)
    notifier.loop()


def main():
    """
    Main entry point.
    Handle command line arguments and start the converter
    """
    # Handle arguments
    parser = argparse.ArgumentParser(
        description='RDF to HTML converter. Converts one or more RDF files '
                    'into a more human readable HTML representation.',)
    parser.add_argument('dcat_files', metavar='DCAT_FILE', type=str, nargs='+',
                        help='DCAT file(s)')
    parser.add_argument('output', metavar='OUTPUT_DIR', type=str,
                        help='Output directory')
    parser.add_argument('--languages', type=str, default='all',
                        help='Languages (on ISO-369-* format) to generate '
                             'separated by comma (,). If omitted all '
                             'encountered languages are generated.')
    parser.add_argument('--watch', action='store_true', help='Watch input '
                        'files for changes and run the conversion when a '
                        'change occurs.')
    parser.add_argument('--verbose', action='store_true',
                        help='Set log level to INFO instead of WARNING')
    parser.add_argument('--log-file', metavar='LOG_FILE',
                        help='File to log to. If omitted logging '
                             'will be sent to stdout')

    args = parser.parse_args()

    setup_logging(args.verbose, args.log_file)

    langs = args.languages.split(',')

    if args.watch:
        watch(args.dcat_files, args.output, langs)
    else:
        for dcat_file in args.dcat_files:
            run(dcat_file, args.output, langs)


def setup_logging(verbose, log_file):
    """
    Setup logging
    """

    if verbose:
        log_level = logging.DEBUG
    else:
        log_level = logging.WARN

    if log_file:
        stream = open(log_file, 'a')
    else:
        stream = sys.stdout

    logging.basicConfig(stream=stream, level=log_level,
                        format='%(asctime)s %(levelname)s '
                               '%(module)s %(message)s')


if __name__ == '__main__':
    main()
